clear all
close all

% This mkcoord program was written for the Redmond Minerals project Line 1, where there were 119 geophones 5m apart
% and 31 total shot points 20m apart starting at 0m, 15m, 35m, ..., 575 m, 595m, then later converted to feet. The 
% reciprocity function has been removed and the times have not been shifted.

% Load the elevation variables. 
load elev_feet.txt
offset=elev_feet(:,1);  % distance is column one.
elevation=elev_feet(:,2);  % elevation is column two.
elevation=-min(elevation)+elevation;  % minimum elevation value must be changed from its absolute value to zero.
SX=[0,49.215:65.620:1886.575,1952.195]'; 	% shot position (ft)
SZ=spline(offset,elevation,SX); % shot elevation (ft)
SN=[1:31]'; 		% shot number
RN=[1:119]';   	% receiver number
RX=[0:16.405:1935.790]'; 	% receiver position (ft)
RZ=spline(offset,elevation,RX);   % receiver elevation (ft)


% Plot the elevations.
figure(1);
subplot(2,1,1)
plot(SX,SZ)
hold on
plot(offset,elevation,'r*')
title('Cubic Spline Interpolation of Relative Elevations (Vertical Exaguration)')
xlabel('Offset (ft)')
ylabel('Relative Elevation (ft)')
subplot(2,1,2)
plot(SX,SZ)
hold on
plot(offset,elevation,'r*')
title('Cubic Spline Interpolation of Relative Elevations (No Vertical Exageration)')
xlabel('Offset (ft)')
ylabel('Relative Elevation (ft)')
axis equal
print -djpeg100 elevation.jpg


% Load the traveltime picks from Vista's .fbp file.
load Line1.fbp  % these two lines need to be renamed according to the name of your .fbp file
FBP=Line1;
sn=FBP(:,2);  	% shot number (column 2)
rn=FBP(:,5);  	% receiver number (column 5)
tt=FBP(:,6);	% traveltime(ms)(column 6)


% Convert pick time units from milliseconds to seconds.
TT=tt/1000;	% traveltime in seconds


% Zero all negative time picks.
for i=1:length(TT)
	if TT(i)<0   % if picks in the traveltime matrix are less than zero...
		TT(i)=0;  % set them equal to zero
	end
end


% Make the traveltime matrix.  The shot number is the horizontal axis and the receiver number is the vertical axis.
SNM=max(sn);
RNM=max(rn);
ttmat=zeros(RNM,SNM);  % begin with a matrix of all zeros...
for i=1:length(TT)     % and fill in the matrix with .fbp data
	ttmat(rn(i),sn(i))=TT(i);
end


% Generate an image of the traveltime matrix.
figure;imagesc(ttmat);colorbar;axis image;title('Traveltime Matrix')
xlabel('Source Number');ylabel('Receiver Number')
print -djpeg100 traveltime.jpg


% This creates an index matrix of ones and changes all points in the matrix that doesn't contain data to zero.
% This index matrix is multiplied by the traveltime matrix to zero out data you want to get rid of.  It is
% most helpful if there are reciprocity errors.
index=ones(119,31); % receiver, shot matrix
tt=ttmat;
for i=1:119  % number of receivers
   for j=1:31  % number of shots
      if tt(i,j)==0 
      index(i,j)=0;
      end
   end
end


% Plot the traveltime matrix after reciprocity rejection (for our data set this plot would be equal to the
% traveltime matrix plotted above.
figure;imagesc(tt);colorbar;axis image;
title('Traveltime Matrix After Reciprocity Rejection')
xlabel('Source Number');ylabel('Receiver Number')
print -djpeg100 traveltime_new.jpg


% Plot the parts of the matrix that either didn't have data or data was rejected.
reject=SNM*RNM - sum(sum(index));
perc_rej=reject*100/(SNM*RNM)
figure;c=[0 0 0;1 1 1];imagesc(index);colormap(c);axis image
title('Rejected Traveltime Picks (Black)')
xlabel('Source Number');ylabel('Receiver Number')
print -djpeg100 rejected.jpg


% Determine the number of traveltimes per shot.
num_rec=sum(index,1)';
figure;plot(num_rec,'b*');title('Number of Traveltime picks per shot')
xlabel('Shot Number');ylabel('Number of Picks')
print -djpeg100 picks_shot.jpg


% Make "coord.temp" source values.
src_num=[1:SNM]';	% Source number 
SOURCE=[src_num SX SZ num_rec];


% Make "coord.temp" receiver values.
rx=RX;
rz=RZ;
count=1;
for i=1:SNM
for j=1:RNM
        if index(j,i)==1
                rec_x(count)=rx(j);
                rec_z(count)=rz(j);
                rec_tt(count)=ttmat(j,i);
                pq(count)=1;
                count=count+1;
        end
end
end
RECEIVER=[rec_x' rec_z' rec_tt' pq'];      


% Make header values
xmin=0;
xmax=max(SX);
zmin=min(SZ);


% Set the depth of the model.
zmax=120;			% total depth of model in meters. This line changes depending on the data set
emax=max(SZ);  	% elevation change over line 
nmr=max(RN);
HEADER=[SNM xmin xmax xmax-xmin zmin
	zmax emax nmr 0 0];


% Write and save "coord.temp".
fid=fopen('coord.temp','w');
fprintf(fid,'%5d %5.1f %5.1f %5.1f %5.1f\n',HEADER(1,:)');
fprintf(fid,'%5.3f %5.1f %5d\n',HEADER(2,1:3)');
fprintf(fid,'%5d %5.2f %5.2f %5d\n',SOURCE');
fprintf(fid,'%5.2f %5.2f %5.5f %5.1f\n',RECEIVER');
fclose(fid);
	

