function [filtered] = harmon(data,freq1,freq2,inter,deltat)
%  function [filtered] = harmon(data,freq1,freq2,inter,deltat)
%
%  Harmon is a time domain notch filter.  Discrete Fourier
%  Transforms are calculated to estimate noise phase and amplitude.
%  The estimated noise is subtracted from the trace in the time domain.
%
%  Usage:
%
%  	Input:
%		data :  Matrix of data to filter
%		freq1 : Start scanning at this frequency
%		freq2 : Upper frequency limit
%		inter : Frequency step (Hz)
%		deltat : Sample rate
%
%	Output:
%		Filtered : Matrix of filtered data
%
%	Example :
%		This is what I use to remove 60 Hz noise
%		filtered = harmon(data,59.88,60.12,0.01,0.0001);
%
%DSIsoft Version 1.0
%written by:
%  Erick ADAM
%  Geological Survey of Canada
%  Email: adam@cg.NRCan.gc.ca
%  Dec. 1995

%Copyright (C) 1998 Seismology and Electromagnetic Section/
%Continental Geosciences Division/Geological Survey of Canada
%
%This library is free software; you can redistribute it and/or
%modify it under the terms of the GNU Library General Public
%License as published by the Free Software Foundation; either
%version 2 of the License, or (at your option) any later version.
%
%This library is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%Library General Public License for more details.
%
%You should have received a copy of the GNU Library General Public
%License along with this library; if not, write to the
%Free Software Foundation, Inc., 59 Temple Place - Suite 330,
%Boston, MA  02111-1307, USA.
%
%DSI Consortium
%Continental Geosciences Division
%Geological Survey of Canada
%615 Booth St.
%Ottawa, Ontario
%K1A 0E9
%
%email: dsi@cg.nrcan.gc.ca

disp('[filtered] = harmon(data,freq1,freq2,inter,deltat)')

freq = freq1:inter:freq2;
freqs = length(freq);
samples = size(data,1);
traces = size(data,2);
cos_wt = ones(samples,freqs);
sin_wt = ones(samples,freqs);
filtered = ones(samples,traces);
samp = (0.0 : deltat : (samples-1)*deltat)';
% Optimization : I create a lookup table for Sine and cosine.
for i = 1 : freqs,
	w = 6.283185307 * freq(i);
	cos_wt(:,i)=cos(samp*w);
	sin_wt(:,i)=sin(samp*w);
end
% Noise estimation
for i = 1 :traces,
	large= 0.0;
	for j = 1: freqs,
		suma = sum(data(:,i) .* cos_wt(:,j))* 2.0 /samples;
		sumb = sum(data(:,i) .* sin_wt(:,j))* 2.0 /samples;
		amp_max = (suma * suma) + (sumb*sumb);
		if(amp_max > large)
			freqn=freq(j);
			freqi=j;
			large = amp_max;
			a_max=suma;
			b_max=sumb;
		end
	end
% The filter is applied here
	filtered(:,i) = data(:,i)-((a_max.*cos_wt(:,freqi))+(b_max .* sin_wt(:,freqi)));
end


