function mat2segy(datain,filename,crosswire,platform,reel_id_header)

%mat2segy(datain,filename,crosswire,platform,reel_id_header)
%
%This function stores a DSI variable in segy format, the oil industry standard
%for seismic data exchange.
%
%This module has been written to use a crosswire file to deliniate
%trace header locations as there are many variations on the segy format.
%This is the same crosswire file as used for 'segy2mat'.
%The crosswire file is and ASCII file consisting of numbers only separted
%into columns using either spaces or Tabs.  MATLAB-style comments can be
%included in the file provided that they begin with a percent character '%'.
%The file should consist of 5 columns: segy byte to start at (byte 1 is
%beginning of a trace header), the format of this element (1 for 'char',
%2 for 'int16', and 3 for 'int32'), a divisor, a value to subtract from the
%element before storing in the segy file, and the trace header word where
%this value is stored in the DSI variable.
%
%See the following paper for a description of the standard segy format:
%Barry, K.M., Cavers, D.A., and Kneale, C.W., Recomended Standards for
%Digital Tape Formats, Geophysics, Vol. 40, No. 2 (April 1975), P. 344-352.
%
%INPUT
%datain - seismic data in DSI format
%filename - name of segy file.  Must be enclosed in single quotes.
%crosswire - name of crosswire file.  Must be enclosed in single quotes.
%platform =	'l' for little-endian (PC) files
%		'b' for big-endian (Unix) files
%reel_id_header - string variable containing a description of the file
%	this string will be stored in the reel identification header of
%	the segy file
%
%DSISoft Customized VSP Processing software
%Version 1.0
%written by K.S. Beaty June, 1998
%last modified June 11, 1998

%Copyright (C) 1998 Seismology and Electromagnetic Section/
%Continental Geosciences Division/Geological Survey of Canada
%
%This library is free software; you can redistribute it and/or
%modify it under the terms of the GNU Library General Public
%License as published by the Free Software Foundation; either
%version 2 of the License, or (at your option) any later version.
%
%This library is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%Library General Public License for more details.
%
%You should have received a copy of the GNU Library General Public
%License along with this library; if not, write to the
%Free Software Foundation, Inc., 59 Temple Place - Suite 330,
%Boston, MA  02111-1307, USA.
%
%DSI Consortium
%Continental Geosciences Division
%Geological Survey of Canada
%615 Booth St.
%Ottawa, Ontario
%K1A 0E9
%
%email: dsi@cg.nrcan.gc.ca

disp('mat2segy(datain,filename,crosswire,platform,reel_id_header)')

platform=lower(platform);
EBCDICSIZE=3200; %EBCDIC/ASCII header length (bytes)
RHLEN=400; %reel header length (bytes)
THLEN=240; %trace header length (bytes)
filler=0;

% open the files
fid=fopen(filename,'w',platform);
if fid== -1
 error('Unable to open data file');
 return;
end %if
fseek(fid,0,-1);

crs=load(crosswire);
for c=1:size(crs,1)
 if crs(c,2)==1
  crsformat{c}='char';
 elseif crs(c,2)==2
  crsformat{c}='int16';
 elseif crs(c,2)==4
  crsformat{c}='float32';
 else
  crsformat{c}='int32'; 
 end %if
end %for

if length(reel_id_header)>EBCDICSIZE
 error('Reel_id_header must contain fewer than ',EBCDICSIZE,' characters.')
end %if

%write reel id header 1
fwrite(fid,reel_id_header,'char');
for i=ftell(fid)+1:EBCDICSIZE
 fwrite(fid,char(32),'char');
end %for
disp('1st reel header written');

%write reel header 2 (file header)
fwrite(fid,filler,'int32');
if ~isempty(datain.fh{2})
fwrite(fid,datain.fh{2},'int32'); %line number (3204)
end %if
if ~isempty(datain.fh{3})
fwrite(fid,datain.fh{3},'int32'); %reel number (3208)
end %if
if ~isempty(datain.fh{13})
fwrite(fid,datain.fh{13},'int16'); %number of traces per record (3212)
end %if
fwrite(fid,filler,'int16');
if ~isempty(datain.fh{8})
fwrite(fid,datain.fh{8}*1000000,'int16'); %sampling rate in microseconds (3216)
end %if
fwrite(fid,filler,'int16');
if ~isempty(datain.fh{7})
npts=datain.fh{7};
fwrite(fid,npts,'int16'); %number of points per trace (3220)
else
error('No information about the number of points per trace, transform abotive!');
return
end %if
fwrite(fid,filler,'int16');
fwrite(fid,2,'int16'); %use format 2 ('int32') for storing traces (3224)
dformat='int32';
dformatlen=4; %4 bytes
fwrite(fid,zeros(1,14),'int16');
fwrite(fid,1,'int16'); %indicates measurement units are metres (3254)
fwrite(fid,zeros(1,172),'int16');

TRLEN=npts*dformatlen; %number of bytes taken up by each trace (data only)

disp('2nd reel header written');

[y,ind]=sort(crs(:,1));
nrec=datain.fh{12};
tr=0;
for rec=1:nrec
 deadtr=find(datain.th{rec}(6,:)<0);
 datain.th{rec}(6,deadtr)=2; %convert DSI kill flags to segy kill flags
 ntr=datain.th{rec}(12,1);
 for k=1:ntr
  %write trace header
  buff=EBCDICSIZE+RHLEN+THLEN*(k-1+tr)+TRLEN*(k-1+tr);

  %use crosswire file to write out trace headers
  for i=1:size(crs,1)
   c=ind(i); %need to write data in order of ascending byte number
   for byteno=ftell(fid)+1:2:buff+crs(c,1)-1
     fwrite(fid,filler,'int16');
   end %for
   value=datain.th{rec}(crs(c,5),k)/crs(c,3)-crs(c,4); %(byte buff+crs(c,1)-1)
   if ~isempty(value)      % make sure value is not empty for 'int' option
   fwrite(fid,value,crsformat{c});
   end %if
  end %for
  buff=buff+THLEN;
  for byteno=ftell(fid)+1:2:buff
    fwrite(fid,filler,'int16');
  end %for

  %write trace data
  fwrite(fid,datain.dat{rec}(:,k),dformat);
 end %loop over traces
 tr=tr+ntr; %number of traces read so far
end %loop over records

fclose(fid);
disp('done writing segy file')
